{*******************************************}
{   Signal Tools 1.0 DSP Components	    }
{    					    }
{   Types: TMarkRecord                      }
{                                           }
{   Components: TMarkCursorEdit             }
{                                           }
{   Copyright (c) 1997-2001 Dew Research    }
{                 Janez Makovsek            }
{   All Rights Reserved                     }
{*******************************************}
{$IMPORTEDDATA ON}
unit SpectrumMarkToolEdit;
{$I BdsppDefs.inc}
interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, Buttons, ComCtrls, SignalToolsTee;

type
  TSpectrumMarkToolEditor = class(TForm)
    MarkPageControl: TPageControl;
    TabSheet1: TTabSheet;
    ClearPeaksButton: TSpeedButton;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    SpectrumBox: TComboBox;
    MarkSeriesBox: TComboBox;
    MarkModeBox: TComboBox;
    CoordinatesBox: TCheckBox;
    ActiveBox: TCheckBox;
    TabSheet2: TTabSheet;
    GroupBox1: TGroupBox;
    AmplitudeEdit: TEdit;
    Label5: TLabel;
    Label6: TLabel;
    PhaseEdit: TEdit;
    Label7: TLabel;
    FrequencyEdit: TEdit;
    LabelHeadersBox: TCheckBox;
    Label4: TLabel;
    MarkTypeBox: TComboBox;
    TabSheet3: TTabSheet;
    DoubleClickClear: TCheckBox;
    FilterBox: TCheckBox;
    PeakZoomBandBox: TCheckBox;
    FilterPeaksButton: TSpeedButton;
    procedure DoubleClickClearClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure SpectrumBoxChange(Sender: TObject);
    procedure MarkSeriesBoxChange(Sender: TObject);
    procedure MarkModeBoxChange(Sender: TObject);
    procedure CoordinatesBoxClick(Sender: TObject);
    procedure ActiveBoxClick(Sender: TObject);
    procedure ClearPeaksButtonClick(Sender: TObject);
    procedure AmplitudeEditChange(Sender: TObject);
    procedure PhaseEditChange(Sender: TObject);
    procedure FrequencyEditChange(Sender: TObject);
    procedure MarkTypeBoxChange(Sender: TObject);
    procedure FilterBoxClick(Sender: TObject);
    procedure PeakZoomBandBoxClick(Sender: TObject);
    procedure FilterPeaksButtonClick(Sender: TObject);
  private
    { Private declarations }
  public
    MarkTool: TSpectrumMarkTool;
    { Public declarations }
  end;

var
  SpectrumMarkToolEditor: TSpectrumMarkToolEditor;

implementation

uses TeEngine, TeeConst;

{$R *.dfm}

procedure TSpectrumMarkToolEditor.DoubleClickClearClick(Sender: TObject);
begin
   MarkTool.DoubleClickClear := DoubleClickClear.Checked;
end;

procedure TSpectrumMarkToolEditor.FormShow(Sender: TObject);
begin
  MarkTool := TSpectrumMarkTool(Tag);
  with MarkTool, SpectrumBox do
  begin
     FillSeriesItems(Items,ParentChart);
     Enabled := ParentChart.SeriesCount>0;
     Items.InsertObject(0,TeeMsg_None,nil);
     ItemIndex := Items.IndexOfObject(Series);
  end;
  With MarkTool, MarkSeriesBox do
  begin
     FillSeriesItems(Items,ParentChart);
     Enabled := ParentChart.SeriesCount>0;
     Items.InsertObject(0,TeeMsg_None,nil);
     ItemIndex := Items.IndexOfObject(MarkSeries);
  end;
  MarkModeBox.ItemIndex := Integer(MarkTool.MarkMode);
  MarkTypeBox.ItemIndex := Integer(MarkTool.MarkType);
//  CoordinatesBox.Checked := MarkTool.DisplayCoordinates;
  FilterBox.Checked := MarkTool.PeakFilterMode;
  ActiveBox.Checked := MarkTool.CursorActive;
  AmplitudeEdit.Text := MarkTool.AmpltFormat;
  PhaseEdit.Text := MarkTool.PhaseFormat;
  FrequencyEdit.Text := MarkTool.FreqFormat;
  LabelHeadersBox.Checked := MarkTool.LabelHeaders;
  PeakZoomBandBox.Checked := MarkTool.PeakZoomBand;
  DoubleClickClear.Checked := MarkTool.DoubleClickClear;
end;

procedure TSpectrumMarkToolEditor.SpectrumBoxChange(Sender: TObject);
begin
  With SpectrumBox do MarkTool.Series:=TChartSeries(Items.Objects[ItemIndex]);
end;

procedure TSpectrumMarkToolEditor.MarkSeriesBoxChange(Sender: TObject);
begin
    With MarkSeriesBox do MarkTool.MarkSeries:=TChartSeries(Items.Objects[ItemIndex]);
end;

procedure TSpectrumMarkToolEditor.MarkModeBoxChange(Sender: TObject);
begin
    MarkTool.MarkMode := TSpectrumMarkMode(MarkModeBox.ItemIndex);
end;

procedure TSpectrumMarkToolEditor.CoordinatesBoxClick(Sender: TObject);
begin
//    MarkTool.DisplayCoordinates := CoordinatesBox.Checked;
end;

procedure TSpectrumMarkToolEditor.ActiveBoxClick(Sender: TObject);
begin
     MarkTool.CursorActive :=  ActiveBox.Checked;
end;

procedure TSpectrumMarkToolEditor.ClearPeaksButtonClick(Sender: TObject);
begin
     MarkTool.ClearMarks;
end;

procedure TSpectrumMarkToolEditor.AmplitudeEditChange(Sender: TObject);
begin
    MarkTool.AmpltFormat := AmplitudeEdit.Text;
end;

procedure TSpectrumMarkToolEditor.PhaseEditChange(Sender: TObject);
begin
    MarkTool.PhaseFormat := PhaseEdit.Text;
end;

procedure TSpectrumMarkToolEditor.FrequencyEditChange(Sender: TObject);
begin
     MarkTool.FreqFormat := FrequencyEdit.Text;
end;

procedure TSpectrumMarkToolEditor.MarkTypeBoxChange(Sender: TObject);
begin
        MarkTool.MarkType := TSpectrumMarkType(MarkTypeBox.ItemIndex);
end;

procedure TSpectrumMarkToolEditor.FilterBoxClick(Sender: TObject);
begin
      MarkTool.PeakFilterMode := FilterBox.Checked;
end;

procedure TSpectrumMarkToolEditor.PeakZoomBandBoxClick(Sender: TObject);
begin
      MarkTool.PeakZoomBand := PeakZoomBandBox.Checked;
end;

procedure TSpectrumMarkToolEditor.FilterPeaksButtonClick(Sender: TObject);
begin
     MarkTool.FilterPeaks;
end;

initialization
RegisterClass(TSpectrumMarkToolEditor);

end.
